/*
 * DiaCanvasElement
 *
 * This is LGPL'ed code.
 */

#include "dia-canvas-element.h"
#include "dia-handle.h"
#include "dia-shape.h"
#include <libart_lgpl/art_affine.h>
#include <math.h>
#include "dia-canvas-i18n.h"

enum {
	PROP_WIDTH = 1,
	PROP_HEIGHT,
	PROP_MIN_WIDTH,
	PROP_MIN_HEIGHT,
	PROP_CNX_LIST,	/*La lista de ángulos de conexión,  Y ESCRITURA*/
        PROP_N_CNX,	/*Cantidad de puntos de conexión DE LECTURA*/
	PROP_REZI	/*the object can be resize? true/false ¿cambio de tamaño?*/
	
};

static void dia_canvas_element_class_init	(DiaCanvasElementClass *klass);
static void dia_canvas_element_init		(DiaCanvasElement *item);
static void dia_canvas_element_set_property	(GObject *object,
						 guint property_id,
						 const GValue *value,
						 GParamSpec *pspec);
static void dia_canvas_element_get_property	(GObject *object,
						 guint property_id,
						 GValue *value,
						 GParamSpec *pspec);
static void dia_canvas_element_update		(DiaCanvasItem *item,
						 gdouble affine[6]);
static gdouble dia_canvas_element_point		(DiaCanvasItem *item,
						 gdouble x, gdouble y);
static void dia_canvas_element_handle_motion	(DiaCanvasItem *item,
						 DiaHandle *handle,
						 gdouble *wx, gdouble *wy,
						 DiaEventMask mask);
static gdouble dia_canvas_element_glue		(DiaCanvasItem *item,
						 DiaHandle *handle,
						 gdouble *x, gdouble *y);
static gboolean dia_canvas_element_connect	(DiaCanvasItem *item,
						 DiaHandle *handle);
static gboolean dia_canvas_element_disconnect	(DiaCanvasItem *item,
						 DiaHandle *handle);


/*
 * relative_angle_from:
 * @gdouble: x,
 * @gdouble: y
 * Entrega el ángulo relativo, de acuerdo 
 * a los parámetros entregados
 * Las coordenadas  recibidas tienen que ser relativas
 * Entrega un valor entero con el ángulo 0-360 grados
 *
*/
static int dia_canvas_element_relative_angle_from(double x, double y);

/*
 * dia_canvas_relative_hypotenusa_from:
 * @gdouble: x,
 * @gdouble: y
 * Entrega el valor de la hipotenusa, de acuerdo 
 * a los parámetros entregados
 * Las coordenadas  recibidas tienen que ser relativas
 * Entrega un valor double 
 *
*/
static double dia_canvas_relative_hypotenusa_from(double x, double y);


/*
 * relocalize_point_from:
 * @DiaCanvasItem: *item
 * @gdouble *x
 * @gdouble *y
 * Recibe  un par de puntos en coordenadas absolutas
 *   y los cambia de acuerdo a las coordenadas del ítem
 * traslación del origen del plano de coordenadas
*/
static void relocalize_point_from(DiaCanvasItem *item,gdouble *x,gdouble *y);


/*En el momento de existir algún cambio de
  tamaño del objeto en cuestión
  se ha de actualizar la posición de los 
  ángulos, manteniendo el valor  del
  ángulo inicial
*/

 
/*puts the new angle connex on list*/
static void save_dia_angleconex_point(DiaCanvasItem *item,DiaAngleConex* punto_conxe);
/*
 * guarda la desviación o tolerancia de un punto de conex dado
 */
static void save_dia_angleconex_point_tol(DiaAngleConex* punto_conxe, int toleran);

/*
 *borra un DiaAngleConex* según ángulo de conexión
 * real
 **/
static void del_dia_angle_conex_point_r(DiaCanvasItem *item,int ang);

/*
 *borra un DiaAngleConex* según ángulo de conexión
 * usuario
 **/
static void del_dia_angle_conex_point_u(DiaCanvasItem *item,int ang);


/*
 * Actualiza los ángulos cuando al objeto se le 
 * cambia de tamaño
 * Sólo los ángulos de referencia
*/
static void update_dia_element_angle_conex(DiaCanvasItem* item); 

/*
 *Entrega el DiaAngleConex de acuerdo al ángulo
 * real que se pase como parámetro
 */
static DiaAngleConex* get_dia_angle_conex_r(DiaCanvasItem *item,int ang);


/*
 *Entrega el DiaAngleConex de acuerdo al ángulo
 * de usuario que se pase como parámetro.
 * el ángulo de usuario es aquel que 
 * el usario puso en un principio y que no cambia aún cambie el tamaño
 */
static DiaAngleConex* get_diaangle_conex_u(DiaCanvasItem *item,int ang);

/*
 *Obtiene la lista de puntos de conexión to the item
 * Return the GList conexion_points
 */
static GList* get_dia_angle_conex_list(DiaCanvasItem* Item);


/*
 *  «Guarda» o pone la lista de puntos de conexión to the item
 *  «saves» or «puts» the GList conexion_points
 */
static GList* save_dia_angle_conex_list(DiaCanvasItem* Item);


/*
 *Busca sí un ángulo existe o no
 */
static int find_angle_conex_tolerantia_u(DiaCanvasItem *item, int ang,int tol);

/*guarda la lista de conexión del ítem
 */
static void dia_canvas_element_save_conex_list(DiaCanvasItem *item,GList* list);



static DiaAngleConex* get_diaangle_conex_u(DiaCanvasItem *item,int ang);

/* si un determinado ángulo s epuede conectar o no
 * en un lugar real.
 */
static DiaAngleConex* get_dia_angle_conex_r(DiaCanvasItem *item,int ang);

/*
 *busca si un ángulo x, se encuentra dentro de la tolerancia de
 * de un ángulo y de conexión de in determinado ítem
 * en una pos real.
 */

static int find_angle_conex_tolerantia_r(DiaCanvasItem *item, int ang);


/*
 * entrega el flujo de datos
 * según el ángulo dado
 * Busca en los ángulos absolutos y no en los
 * relativos.
 * La dirección en que se puede conectar la línea, sí es entrada o salida
*/
static int find_conex_fluxus(DiaCanvasItem *item, int ang);
 

/*actualiza el centro geométrico*/
static void update_center_point (DiaCanvasItem *item);



/* entrea el ángulo of de conex, según la tolerancia*/
static int find_angle_conex_ref_from_tol(DiaCanvasItem *item, int ang);

#define align_handles(item) \
	dia_canvas_element_align_handles ((DiaCanvasElement*) item)

#define GET_HANDLE(item, dir) (DIA_HANDLE (g_list_nth_data (((DiaCanvasItem*) (item))->handles, dir)))
#define GET_HANDLE_DIR(item, handle) (g_list_index (((DiaCanvasItem*) (item))->handles, handle))

static DiaCanvasItemClass *parent_class = NULL;

GType
dia_canvas_element_get_type (void)
{
	static GType object_type = 0;

	if (!object_type) {
		static const GTypeInfo object_info = {
			sizeof (DiaCanvasElementClass),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			(GClassInitFunc) dia_canvas_element_class_init,
			(GClassFinalizeFunc) NULL,
			(gconstpointer) NULL, /* class_data */
			sizeof (DiaCanvasElement),
			(guint16) 0, /* n_preallocs */
			(GInstanceInitFunc) dia_canvas_element_init,
		};

		object_type = g_type_register_static (DIA_TYPE_CANVAS_ITEM,
						      "DiaCanvasElement",
						      &object_info,
						      G_TYPE_FLAG_ABSTRACT);
	}

	return object_type;
}


static void
dia_canvas_element_class_init (DiaCanvasElementClass *klass)
{
	GObjectClass *object_class;
	DiaCanvasItemClass *item_class;
	DiaCanvasElementClass *element_class;

	object_class = (GObjectClass*) klass;
	item_class = DIA_CANVAS_ITEM_CLASS (klass);
	element_class=DIA_CANVAS_ELEMENT_CLASS (klass);

	parent_class = g_type_class_peek_parent (klass);

	object_class->get_property = dia_canvas_element_get_property;
	object_class->set_property = dia_canvas_element_set_property;
	
	g_object_class_install_property (object_class,
					 PROP_WIDTH,
					 g_param_spec_double ("width",
						 _("Width"),
						 _("Width of the element"),
						 0.0, G_MAXDOUBLE, 1.0,
						 G_PARAM_READWRITE));
	g_object_class_install_property (object_class,
					 PROP_HEIGHT,
					 g_param_spec_double ("height",
						 _("Height"),
						 _("Height of the element"),
						 0.0, G_MAXDOUBLE, 1.0,
						 G_PARAM_READWRITE));
	g_object_class_install_property (object_class,
					 PROP_MIN_WIDTH,
					 g_param_spec_double ("min_width",
						 _("Minimal width"),
						 _("Minimal width of the element"),
						 0.0, G_MAXDOUBLE, 1.0,
						 G_PARAM_READWRITE));
	g_object_class_install_property (object_class,
					 PROP_MIN_HEIGHT,
					 g_param_spec_double ("min_height",
						 _("Minimal height"),
						 _("Minimal height of the element"),
						 0.0, G_MAXDOUBLE, 1.0,
						 G_PARAM_READWRITE));
/**
  * @AN_ADVICE:IS GOOD THAT THE FOLLOWING PARAMETER PROPERTIES
  *             SET BY HIS DIA-CANVAS-ITEM PROPER FUNCTIONS   
  **/
       /*
        * Objeto de tipo GList, que referencia 
        * la lista de ángulos en los cuales 
        * se puede conectar la línea al objeto
          */
	g_object_class_install_property (object_class,
					 PROP_CNX_LIST,
					 g_param_spec_boxed ("angulus_list",
						 _("Lista de puntos de conex"),
					 _("Inserts Authorized list conexion point"),
					 G_TYPE_BOXED,
					 G_PARAM_READWRITE));
        /*si el objeto puede cambiar de tamaño
         * o no
         */   
	g_object_class_install_property (object_class,
					 PROP_REZI,
					 g_param_spec_boolean ("resizable",
						 _("Resizable"),
					 _("Whether the canvas item is resizable"),
					 TRUE,
					 G_PARAM_READWRITE));
	item_class->update = dia_canvas_element_update;
	item_class->point = dia_canvas_element_point;
	item_class->handle_motion = dia_canvas_element_handle_motion;
	item_class->glue = dia_canvas_element_glue;
	item_class->connect = dia_canvas_element_connect;
	item_class->disconnect = dia_canvas_element_disconnect;

        element_class->relative_angle_from=dia_canvas_element_relative_angle_from;
	element_class->relative_hypotenusa_from=dia_canvas_relative_hypotenusa_from;
	element_class->relocalize_point_from=relocalize_point_from;

}


static void
dia_canvas_element_init (DiaCanvasElement *item)
{
	int i;

	item->width = 100.0;
	item->height = 100.0;
	item->min_width = 10.0;
	item->min_height = 10.0;
	for (i = 0; i < 8; i++) {
		g_object_new (DIA_TYPE_HANDLE,
				"owner", item,
				"strength", DIA_STRENGTH_STRONG,
				"movable", TRUE,
				NULL);
	}
	//DIA_SET_FLAGS ((DiaCanvasItem*)item, DIA_NEED_ALIGN_HANDLES);
        item->resizable=TRUE;  	
	align_handles (item);
}


static void dia_canvas_element_set_property (GObject *object, guint property_id,
				 const GValue *value, GParamSpec *pspec)
{
	DiaCanvasElement *element = (DiaCanvasElement*) (object);
     /*added*/
	DiaCanvasItem* item = (DiaCanvasItem*) (object);
	
	switch (property_id) {
	case PROP_WIDTH:
		dia_canvas_item_preserve_property (&element->item, "width");
		element->width = g_value_get_double (value);
		align_handles (DIA_CANVAS_ITEM (element));
		dia_canvas_item_request_update (DIA_CANVAS_ITEM (element));
		break;
	case PROP_HEIGHT:
		dia_canvas_item_preserve_property (&element->item, "height");
		element->height = g_value_get_double (value);
		align_handles (DIA_CANVAS_ITEM (element));
		dia_canvas_item_request_update (DIA_CANVAS_ITEM (element));
		break;
	case PROP_MIN_WIDTH:
		dia_canvas_item_preserve_property (&element->item, "min_width");
		element->min_width = g_value_get_double (value);
		if (element->width < element->min_width) {
			dia_canvas_item_preserve_property (&element->item,
							   "width");
			element->width = element->min_width;
			align_handles (DIA_CANVAS_ITEM (element));
			dia_canvas_item_request_update (DIA_CANVAS_ITEM (element));
		}
		dia_canvas_item_request_update (DIA_CANVAS_ITEM (element));
		break;
	case PROP_MIN_HEIGHT:
		dia_canvas_item_preserve_property (&element->item, "min_height");
		element->min_height = g_value_get_double (value);
		if (element->height < element->min_height) {
			dia_canvas_item_preserve_property (&element->item,
							   "height");
			element->height = element->min_height;
			align_handles (DIA_CANVAS_ITEM (element));
			dia_canvas_item_request_update (DIA_CANVAS_ITEM (element));
		}
		break;
        case PROP_CNX_LIST:   /*El ángúlo de conexión*/
		{
		 GList* list;
		 list=(GList*)g_value_get_boxed(value); /*obtain the angle list*/
		 dia_canvas_element_save_conex_list(item,list);
		}break;
        case PROP_REZI:   /*cambia de tamaño*/
		{
                 /* The items collection of handles. */
		 GList *handles, *node;
                 int cant,i;
		 DiaHandle* handle;
		 element->resizable=g_value_get_boolean(value); /*obtain resizable¿?*/
                 handles=DIA_CANVAS_ITEM (element)->handles;
  		 cant=g_list_length(handles); 
		 for(i=0;i<cant;i++)
		 {
		   node=g_list_nth(handles,i);
   		   handle=node->data;
		   g_object_set(handle,"movable",element->resizable,NULL);	
		  }
		 dia_canvas_item_request_update (item);
		}break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
dia_canvas_element_get_property (GObject *object, guint property_id,
				 GValue *value, GParamSpec *pspec)
{
     /*added*/
	DiaCanvasItem* item = (DiaCanvasItem*) (object);
	switch (property_id) {
	case PROP_WIDTH:
		g_value_set_double (value, DIA_CANVAS_ELEMENT (object)->width);
		break;
	case PROP_HEIGHT:
		g_value_set_double (value, DIA_CANVAS_ELEMENT (object)->height);
		break;
	case PROP_MIN_WIDTH:
		g_value_set_double (value, DIA_CANVAS_ELEMENT (object)->min_width);
		break;
	case PROP_MIN_HEIGHT:
		g_value_set_double (value, DIA_CANVAS_ELEMENT (object)->min_height);
		break;
        case PROP_CNX_LIST:   /*El ángúlo de conexión*/
		g_value_set_boxed (value, item->conxpoint);
		break;
	case PROP_REZI:
		g_value_set_boolean(value,DIA_CANVAS_ELEMENT (object)->resizable);
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

/**
 * dia_canvas_element_get_opposite_handle:
 * @item: 
 * @handle: 
 *
 * Return the opposite handle of @handle.
 *
 * Return value: opposite handle.
 **/
DiaHandle*
dia_canvas_element_get_opposite_handle (DiaCanvasItem *item,
					DiaHandle *handle)
{
	int opp = -1;

	switch(GET_HANDLE_DIR(item,handle)){
	case DIA_HANDLE_N:
		opp = DIA_HANDLE_S;
		break;
	case DIA_HANDLE_S:
		opp = DIA_HANDLE_N;
		break;
	case DIA_HANDLE_E:
		opp = DIA_HANDLE_W;
		break;
	case DIA_HANDLE_W:
		opp = DIA_HANDLE_E;
		break;
	case DIA_HANDLE_NW:
		opp = DIA_HANDLE_SE;
		break;
	case DIA_HANDLE_SW:
		opp = DIA_HANDLE_NE;
		break;
	case DIA_HANDLE_NE:
		opp = DIA_HANDLE_SW;
		break;
	case DIA_HANDLE_SE:
		opp = DIA_HANDLE_NW;
		break;
	default:
		g_assert_not_reached ();
	}
	return GET_HANDLE (item, opp);
}

void
dia_canvas_element_align_handles (DiaCanvasElement *element)
{
	DiaCanvasItem *item = DIA_CANVAS_ITEM (element);

	//g_message (G_STRLOC": w %f h %f", el->width, el->height);

	dia_handle_set_pos_i (GET_HANDLE (item, DIA_HANDLE_NW), 0.0, 0.0);

	/* Alter the affine transformation matrix so that the NW handle will
	 * be at (0,0).
	 */
	dia_handle_set_pos_i (GET_HANDLE (item, DIA_HANDLE_N),
			      element->width/2, 0.0);

	dia_handle_set_pos_i (GET_HANDLE (item, DIA_HANDLE_NE),
			      element->width, 0.0);

	dia_handle_set_pos_i (GET_HANDLE (item, DIA_HANDLE_E),
			      element->width, element->height/2);

	dia_handle_set_pos_i (GET_HANDLE (item, DIA_HANDLE_SE),
			      element->width, element->height);

	dia_handle_set_pos_i (GET_HANDLE (item, DIA_HANDLE_S),
			      element->width/2, element->height);

	dia_handle_set_pos_i (GET_HANDLE (item, DIA_HANDLE_SW),
			      0.0, element->height);

	dia_handle_set_pos_i (GET_HANDLE (item, DIA_HANDLE_W),
			      0.0, element->height/2);
}

static void
dia_canvas_element_update (DiaCanvasItem *item, gdouble affine[6])
{
        if (DIA_CANVAS_ITEM_CLASS (parent_class)->update)
		DIA_CANVAS_ITEM_CLASS (parent_class)->update (item, affine);

	/* if (item->flags & DIA_NEED_ALIGN_HANDLES) {
		align_handles (item);
		DIA_UNSET_FLAGS (item, DIA_NEED_ALIGN_HANDLES);
	} */

	item->bounds.left = 0.0;
	item->bounds.top = 0.0;
	item->bounds.right = DIA_CANVAS_ELEMENT (item)->width;
	item->bounds.bottom = DIA_CANVAS_ELEMENT (item)->height;

	update_center_point(item);
	update_dia_element_angle_conex(item);

}


static gdouble
dia_canvas_element_point (DiaCanvasItem *item, gdouble x, gdouble y)
{
	DiaPoint p;

	p.x = x;
	p.y = y;

	return dia_distance_rectangle_point (&item->bounds, &p);
}


/**
 * dia_canvas_element_handle_motion_move:
 * @item: 
 * @handle: 
 * @wx: 
 * @wy: 
 *
 * Included in dia_canvas_element_handle_motion(). Used to move a handle.
 **/
static void
dia_canvas_element_handle_motion_move (DiaCanvasItem *item, DiaHandle *handle,
				       gdouble *wx, gdouble *wy)
{
	DiaCanvasElement *el = (DiaCanvasElement*) item;
	gdouble ix, iy;
	gdouble nw_dx = 0.0, nw_dy = 0.0;
	gdouble x, y;
	gdouble dx, dy; /* The movement relative to the handle */
	gint dir;

	/* Do not align handles if the handle is not part of the first 8
	 * handles. */
	if (GET_HANDLE_DIR (item, handle) > DIA_HANDLE_SE)
		return;

	g_object_freeze_notify (G_OBJECT (item));

	dia_canvas_preserve_property (item->canvas, (GObject*) item, "width");
	dia_canvas_preserve_property (item->canvas, (GObject*) item, "height");

	dia_handle_get_pos_i (handle, &x, &y);
	dir = GET_HANDLE_DIR (item, handle);

	ix = *wx;
	iy = *wy;
	dia_canvas_item_affine_point_w2i (item, &ix, &iy);

	dx = ix - x;
	dy = iy - y;

	/* If the north or west side is moved, the object should be moved to
	 * keep the origin in the upper right corner. */
	if ((dir == DIA_HANDLE_NW) || (dir == DIA_HANDLE_N)
	    || (dir == DIA_HANDLE_NE)) {
		register gdouble se_y = dia_variable_get_value (GET_HANDLE (item, DIA_HANDLE_SE)->pos_i.y);
		if (iy > se_y)
			dy = se_y - y;
		nw_dy = dy;
		dy = -dy;
	}
	if ((dir == DIA_HANDLE_NW) || (dir == DIA_HANDLE_W)
	    || (dir == DIA_HANDLE_SW)) {
		register gdouble se_x = dia_variable_get_value (GET_HANDLE (item, DIA_HANDLE_SE)->pos_i.x);
		if (ix > se_x)
			dx = se_x - x;
		nw_dx = dx;
		dx = -dx;
	}

	if ((dir != DIA_HANDLE_N) && (dir != DIA_HANDLE_S)) {
		el->width += dx;
		g_object_notify ((GObject*) item, "width");
	}
	if ((dir != DIA_HANDLE_W) && (dir != DIA_HANDLE_E)) {
		el->height += dy;
		g_object_notify ((GObject*) item, "height");
	}

	if (el->width <= el->min_width) {
		if ((dir == DIA_HANDLE_NW) || (dir == DIA_HANDLE_W)
		    || (dir == DIA_HANDLE_SW))
			nw_dx += el->width - el->min_width;
		el->width = el->min_width;
	}
	if (el->height <= el->min_height) {
		if ((dir == DIA_HANDLE_NW) || (dir == DIA_HANDLE_N)
		    || (dir == DIA_HANDLE_NE))
			nw_dy += el->height - el->min_height;
		el->height = el->min_height;
	}
	
	/* Move the entire object if the north or west handle is moved. */
	if ((nw_dx != 0.0) || (nw_dy != 0.0)) {
		gdouble parent_affine[6], a[6];

		dia_canvas_preserve_property (item->canvas, (GObject*) item, "affine");

		dia_canvas_item_affine_w2i (DIA_CANVAS_ITEM (item->parent),
					    parent_affine);
		art_affine_translate (a, nw_dx, nw_dy);

		art_affine_multiply (a, a, parent_affine);
		art_affine_multiply (item->affine, a, item->affine);
		g_object_notify ((GObject*) item, "affine");
	}

	align_handles (item);
	
	dia_handle_get_pos_w (handle, wx, wy);
	g_object_thaw_notify ((GObject*) item);

	update_center_point (item);
}

/**
 * dia_canvas_element_handle_motion_rotate:
 * @item: 
 * @handle: 
 * @wx: 
 * @wy: 
 *
 * This function is included in dia_canvas_element_handle_motion(), to
 * provide rotation for the corner-handles.
 **/
static inline void
dia_canvas_element_handle_motion_rotate (DiaCanvasItem *item, DiaHandle *handle,
					 gdouble *wx, gdouble *wy,
					 gboolean pinned)
{
	gdouble x, y;
	gdouble ix,iy;
	gdouble a, b;
	gdouble alpha, angle;

	dia_handle_get_pos_i (handle, &x, &y);
	ix = *wx;
	iy = *wy;
	dia_canvas_item_affine_point_w2i (item, &ix, &iy);

	/* (a, b) is the center of the object. */
	a = item->bounds.left + (item->bounds.right - item->bounds.left) / 2;
	b = item->bounds.top + (item->bounds.bottom - item->bounds.top) / 2;

	/* if (pinned) {
		gdouble ta = a, tb = b;
		a = item->affine[0] * ta + item->affine[2] * tb;
		b = item->affine[1] * ta + item->affine[3] * tb;
	} */

	alpha = atan2 ((a - ix), (b - iy));

	if (pinned) {
		alpha = (((int) (alpha / M_PI * 180.0)) / 5) * 5;
		//g_message (G_STRLOC": alpha = %f", alpha);
		alpha = alpha * M_PI / 180.0;
	}

	angle = (atan2 ((a - x), (b - y)) - alpha) / M_PI * 180.0;

	dia_canvas_item_rotate (item, angle);

	align_handles (item);
	
	dia_handle_get_pos_w (handle, wx, wy);

	update_center_point (item);
}

/**
 * dia_canvas_element_handle_motion_shear:
 * @item: 
 * @handle: 
 * @wx: 
 * @wy: 
 *
 * Included in dia_canvas_element_handle_motion(). Used to shear the North,
 * South, east and West handles (when CTRL is pressed).
 **/
static inline void
dia_canvas_element_handle_motion_shear (DiaCanvasItem *item, DiaHandle *handle,
					gdouble *wx, gdouble *wy)
{
	gdouble x, y;
	gdouble ix, iy;
	//gdouble tx, ty;
	gdouble affine[6];

	dia_handle_get_pos_i (handle, &x, &y);
	//dia_handle_get_pos_w (handle, &ix, &iy);
	dia_canvas_item_affine_w2i (item, affine);

	ix = *wx;
	iy = *wy;
	dia_canvas_item_affine_point_w2i (item, &ix, &iy);


	/* We need to ignore the x or y variable of the cursor position
	 * (*wx, *wy). Give it the handle's position. */
	switch (GET_HANDLE_DIR (item, handle)) {
	case DIA_HANDLE_N:
		dia_canvas_item_shear_x (item, x - ix, y - iy);
		break;
	case DIA_HANDLE_S:
		dia_canvas_item_shear_x (item, ix - x, iy - y);
		break;
	case DIA_HANDLE_E:
		dia_canvas_item_shear_y (item, ix - x, iy - y);
		break;
	case DIA_HANDLE_W:
		dia_canvas_item_shear_y (item, x - ix, y - iy);
		break;
	default:
		g_assert_not_reached ();
	}
	align_handles (item);
	
	/* Force an i->w update */
	dia_handle_get_pos_w (handle, wx, wy);

	update_center_point (item);
}

/**
 * dia_canvas_element_handle_motion:
 * @item: 
 * @handle: 
 * @wx: 
 * @wy: 
 * @mask: 
 *
 * Move a handle, inlines dia_canvas_element_handle_motion_move(),
 * dia_canvas_element_handle_motion_rotate() and
 * dia_canvas_element_handle_motion_shear().
 **/
static void
dia_canvas_element_handle_motion (DiaCanvasItem *item, DiaHandle *handle,
			          gdouble *wx, gdouble *wy, DiaEventMask mask)
{
	if (mask & DIA_EVENT_MASK_CTRL) {
		guint dir = GET_HANDLE_DIR (item, handle);
		switch (dir) {
		case DIA_HANDLE_NW:
		case DIA_HANDLE_NE:
		case DIA_HANDLE_SE:
		case DIA_HANDLE_SW:
			dia_canvas_element_handle_motion_rotate (item, handle,
					 wx, wy, mask & DIA_EVENT_MASK_SHIFT);
			break;
		case DIA_HANDLE_N:
		case DIA_HANDLE_S:
		case DIA_HANDLE_E:
		case DIA_HANDLE_W:
			dia_canvas_element_handle_motion_shear (item, handle,
								wx, wy);
			break;
		default:
			g_assert_not_reached ();
		}
	} else if (mask & DIA_EVENT_MASK_SHIFT) {
		gdouble hx, hy, ox, oy;
		DiaHandle *opphandle = dia_canvas_element_get_opposite_handle (item, handle);
		dia_handle_get_pos_w (handle, &hx, &hy);
		dia_handle_get_pos_w (opphandle, &ox, &oy);
		ox -= *wx - hx;
		oy -= *wy - hy;
		g_object_freeze_notify ((GObject*) item);
		dia_canvas_element_handle_motion_move (item, handle, wx, wy);
		dia_canvas_element_handle_motion_move (item, opphandle, &ox, &oy);
		g_object_thaw_notify ((GObject*) item);
	} else {
		dia_canvas_element_handle_motion_move (item, handle, wx, wy);
	}

	dia_canvas_item_request_update (item);
	update_center_point (item);
}

/**
 * calc_glue_point:
 * @item: 
 * @handle: 
 * @p: in item coordinates!
 * @segment: First segment, clockwise.
 *
 *
 *
 * Return value: 
 **/
/*calcula o entrega el lugar en dónde haber punto de conexión
*/
static gdouble
calc_glue_point (DiaCanvasItem *item, DiaHandle *handle,
		 DiaPoint *p, gint *segment)
{
	DiaPoint ul, ur, ll, lr;
	DiaPoint pol1, pol2, pol3, pol4;
	gdouble min, d1, d2, d3, d4;
   
	dia_handle_get_pos_i (GET_HANDLE (item, DIA_HANDLE_NW), &ul.x, &ul.y);
	dia_handle_get_pos_i (GET_HANDLE (item, DIA_HANDLE_SE), &lr.x, &lr.y);
	ur.x = lr.x;
	ur.y = ul.y;
	ll.x = ul.x;
	ll.y = lr.y;

	d1 = dia_distance_line_point (&ul, &ur, p, 0.0, DIA_CAP_BUTT, &pol1);
	d2 = dia_distance_line_point (&ur, &lr, p, 0.0, DIA_CAP_BUTT, &pol2);
	d3 = dia_distance_line_point (&ll, &lr, p, 0.0, DIA_CAP_BUTT, &pol3);
	d4 = dia_distance_line_point (&ul, &ll, p, 0.0, DIA_CAP_BUTT, &pol4);

	min = MIN (MIN(d1, d2), MIN(d3, d4));

	//art_affine_invert (inv, a);

	if (min == d1) {
		if (p) *p = pol1;
		if (segment) *segment = DIA_HANDLE_NW;
//                printf("calc_glue_point posición en punto de pegar  d1 %f \n",d1);
		return d1;
	} else if (min == d2) {
		if (p) *p = pol2;
		if (segment) *segment = DIA_HANDLE_NE;
//                printf("calc_glue_point posición en punto de pegar d2 %f \n",d2);
		return d2;
	} else if (min == d3) {
		if (p) *p = pol3;
		if (segment) *segment = DIA_HANDLE_SE;
//                printf("calc_glue_point posición en punto de pegar d3 %f \n",d3);
		return d3;
	} else {
		if (p) *p = pol4;
		if (segment) *segment = DIA_HANDLE_SW;
//                printf("calc_glue_point posición en punto de pegar d4 %f \n",d4);
		return d4;
	}
}

static gdouble
dia_canvas_element_glue (DiaCanvasItem *item, DiaHandle *handle,
			 gdouble *x, gdouble *y)
{
	DiaPoint p;
	gdouble a[6], inv[6], dist;
         
 	dia_canvas_item_affine_w2i (item, a);
	p.x = *x * a[0] + *y * a[2] + a[4];
	p.y = *x * a[1] + *y * a[3] + a[5];

	dist = calc_glue_point (item, handle, &p, NULL);

	art_affine_invert (inv, a);

	*x = p.x * inv[0] + p.y * inv[2] + inv[4];
	*y = p.x * inv[1] + p.y * inv[3] + inv[5];
	return dist;
}

static gboolean
dia_canvas_element_connect (DiaCanvasItem *item, DiaHandle *handle)
{
	//DiaCanvasElement *el = (DiaCanvasElement*) item;
	DiaHandle *hbegin, *hend = NULL;
	gint segment;
	DiaPoint p, p1,p_ang;
        int ang;
        int perm;
        int cant;
	printf("\n");
        g_print ("+ Info por Item %s (%p/%d)\n", g_type_name (G_OBJECT_TYPE (item)),
		 item, G_OBJECT (item)->ref_count);

//       printf("pos del obj x=%f ,y=%f \n",item->affine[4],item->affine[5]);

/*chnage it a two «lines» dowm
//	if (!parent_class->connect (item, handle))
//		return FALSE;
 */	
       dia_handle_get_pos_w (handle, &p.x, &p.y);
       p_ang.x=p.x;
       p_ang.y=p.y;
       relocalize_point_from(item,&p_ang.x,&p_ang.y);
       ang=dia_canvas_element_relative_angle_from(p_ang.x,p_ang.y);

       perm=dia_canvas_element_exits_cnx_point_h(item,ang,handle);
       if(!perm)  /*el ángulo pasado como parámetro, no tiene permiso de conex*/
         return FALSE;
      if (!parent_class->connect (item, handle))
		return FALSE;

	dia_canvas_item_affine_point_w2i (item, &p.x, &p.y);

	calc_glue_point (item, handle, &p, &segment);

	dia_canvas_item_affine_point_i2w (item, &p.x, &p.y);
        

	dia_handle_set_pos_w (handle, p.x, p.y);


	hbegin = GET_HANDLE(item, segment);
//       printf("después de afinar:pos del handle x=%f ,y=%f \n",p.x, p.y);


        /*Los segmentos son vistos desde el punto de vista del usuario, 
          «norte mira hacia arriba»
         */
	switch (segment) {
	case DIA_HANDLE_NW:
		hend = GET_HANDLE (item, DIA_HANDLE_NE);
                printf("pega en norte\n");
		break;
	case DIA_HANDLE_NE:
		hend = GET_HANDLE (item, DIA_HANDLE_SE);
                printf("pega en este\n");
		break;
	case DIA_HANDLE_SE:
		hend = GET_HANDLE (item, DIA_HANDLE_SW);
                printf("pega en sur\n");
		break;
	case DIA_HANDLE_SW:
		hend = GET_HANDLE (item, DIA_HANDLE_NW);
                printf("pega en oeste\n");
		break;
	default:
		g_assert_not_reached ();
	}
	g_assert (hbegin != NULL);
	g_assert (hend != NULL);

	dia_handle_remove_all_constraints (handle);

	dia_handle_add_line_constraint (hbegin, hend, handle);
	return TRUE;
}

static gboolean
dia_canvas_element_disconnect (DiaCanvasItem *item, DiaHandle *handle)
{
	return parent_class->disconnect (item, handle);
}



/*
 * dia_canvas_element_save_conex_list:
 * @DiaCanvasItem: *item
 * @GList *list
 * Recibe  una lista doblemente enlazada
 *   puntos de conexión
 *
*/
static void dia_canvas_element_save_conex_list(DiaCanvasItem* item,GList* list)
{
	
  int cant, i;
  GList* node;
  DiaAngleConex* punto_conx=NULL;
  DiaAngleConex* punto_conx_c;
  node=NULL;
  cant=g_list_length(list); 
  for(i=0;i<cant;i++)
  {
   node=g_list_nth(list,i);
   punto_conx=node->data;
   punto_conx_c=dia_angle_conex_new (punto_conx->angulus,punto_conx->torelantia,punto_conx_c->fluxus);
   punto_conx_c->ro=punto_conx->ro; 
   punto_conx_c->ro_real=punto_conx->ro_real;

 /*obtain the y and x place from ro_real*/
   punto_conx_c->x=punto_conx->x;
 /*obtain the y and y place from ro_real*/
   punto_conx_c->y=punto_conx->y;
 /*original relative points are saved*/ 
   punto_conx_c->yo=punto_conx->yo;
   punto_conx_c->xo=punto_conx->xo;
 /*xmax & ymax orignal saved*/
    punto_conx_c->xmo=punto_conx->xmo;
    punto_conx_c->ymo=punto_conx->ymo;
 /*xmax var & ymasx var saved*/ 
    punto_conx_c->xm=punto_conx->xm;
    punto_conx_c->ym=punto_conx->ym;
 /*guarda el punto de conexión*/
  item->conxpoint=g_list_append(item->conxpoint,punto_conx_c);
 }
}




/*
 * relocalize_point_from:
 * @DiaCanvasItem: *item
 * @gdouble *x
 * @gdouble *y
 * Recibe  un par de puntos en coordenadas absolutas
 *   y los cambia de acuerdo a las coordenadas del ítem
 * traslación del origen del plano de coordenadas
*/
static void relocalize_point_from(DiaCanvasItem *item,gdouble *x,gdouble *y)
 {
//  printf("pos del cero x=%f ,y=%f \n",item->origo.x, item->origo.y);
  *x=*x-item->origo.x;
  *y=(*y-item->origo.y)*(-1);
 }



/*
 * relative_angle_from:
 * @gdouble: x,
 * @gdouble: y
 * Entrega el ángulo relativo, de acuerdo 
 * a los parámetros entregados
 * Las coordenadas  recibidas tienen que ser relativas
 * Entrega un valor entero con el ángulo 0-360 grados
 *
*/
//static int relative_angle_from(double x, double y)
// {
// double ang_r;
//  //entrega vallores entre -pi e pi
//  ang_r=atan2(y,x);
//  ang_r=ang_r*180/M_PI;
//  if(ang_r<0)  /*rango de 0 a -pi*/
//     ang_r=360+ang_r; /*le corrige*/
//  return (int)ang_r;  /*hace un cambio de tipo .. :)*/
// }

/*
 * dia_canvas_relative_hypotenusa_from:
 * @gdouble: x,
 * @gdouble: y
 * Entrega el valor de la hipotenusa, de acuerdo 
 * a los parámetros entregados
 * Las coordenadas  recibidas tienen que ser relativas
 * Entrega un valor double 
 *
*/
static double dia_canvas_relative_hypotenusa_from(double x,double y) 
 {
  if(x<0.0)
     x*=-1;
  if(y<0.0)
     y*=-1;
  x*=x;
  y*=y;
  x=x+y;
  y=sqrt(x);
  return y;
  }

/*guarda un ágnulo de conexión,  su tolerancia y flujo de información
 *esta función debería llamarse 
 * para cuando se desea poner un nuevo ángulo
 * relativo.
 * de conexión para un ítem determinado
 * inserts a new angle conexion point
 * must be used if wants to puts a 
 * conexion point from a Item
 * Se le encuentra el «ro» dependiendo del x e y
 * El ángulo es con respectivo al usuario
 * entrega 0 si fué efectivo, ó algún cód de error
*/
int dia_canvas_element_insert_new_cnx_p(DiaCanvasItem *item, int angle, int tol,int fluxus)
{
 DiaAngleConex* punto_conx;
 int exist;
 update_center_point(item);
 dia_canvas_item_request_update(item) ;
 /*pregunta si el punto en cuestión existe, o en su tolerancia*/
 exist=find_angle_conex_tolerantia_u(item,angle,tol);
//printf("Existe: %d \n",exist);
 if(!exist)
  {
   punto_conx=dia_angle_conex_new (angle,tol,fluxus);
   save_dia_angleconex_point(item,punto_conx);
   }
 return exist;
}


/*Elimina un águlo de conexión y su tolerancia
 * esta función debería llamarse 
 * para cuando se desea retirar un  ángulo
 * de conexión de un ítem determinado
 * el ángulo ddebe ser el relativo. 
 * delete a  angle conexion point
 * must be used if wants to retires a 
 * conexion point from a Item
*/
void dia_canvas_element_del_cnx_p(DiaCanvasItem *item, int angle)
{
   del_dia_angle_conex_point_u(item,angle);
}



/* 
 *Pregunta si en el ángulo pasado como parámetro
 *tiene permiso de conexión.
 * entrega 0 para No y Uno para sí.
 * Incluye el 
 *También entrega 1 cuando no tiene ángulos seleccionados 
 */
int dia_canvas_element_exits_cnx_point_h(DiaCanvasItem *item, int angle,DiaHandle *handle_sagitta)
{
  int res;  
  int fluxus;
  DiaHandle* manejador;   /*El manejador de conexiones*/
  DiaHandle* manejador2;   /*El manejador de conexiones*/
  DiaCanvasItem* elemento=NULL;
  elemento=handle_sagitta->owner;  /*el dueñod del manejador*/
  /*recupera el manejador cabeza*/
  manejador=g_list_first (DIA_CANVAS_ITEM (elemento)->handles)->data;
  /*recupera el manejador de la cola*/
  manejador2=g_list_last(DIA_CANVAS_ITEM (elemento)->handles)->data;
  res=find_angle_conex_tolerantia_r(item,angle);
  fluxus=find_conex_fluxus(item,angle);
  if(res==1 && fluxus==3) /*el resultado es 1 y la dirección cualquiera*/
    return 1;
  else /*res uno, manejador de cabeza igual al de parámetro*/
  if(res==1 && manejador==handle_sagitta && fluxus==1)
    return 1;
  else
  if(res==1 && manejador2==handle_sagitta && fluxus==0)
    return 1;
  return 0;   /*cualquier otra opción es negada*/
 }


/*
 * entrega el ángulo oficial de conexión 
 * sí se le pasa como parámetro un 
 * ángulo que se encuentra en el dominio
*/
int dia_canvas_element_angle_off_segment(DiaCanvasItem *item,int angle_p)
{
  return find_angle_conex_ref_from_tol(item,angle_p);
}

/*Función que entrega el ángulo en el que se encuentra
 * con respecto al ítem
 * de una línea y un cuadro
 * pero se le pasa el manejador de la línea
*/

int dia_canvas_element_line_angle(DiaCanvasItem *item,DiaHandle *handle_sagitta)
{
   DiaPoint p,p_ang;
   int perm;
   dia_handle_get_pos_w (handle_sagitta, &p.x, &p.y);
   p_ang.x=p.x;
   p_ang.y=p.y;
   relocalize_point_from(item,&p_ang.x,&p_ang.y);
   return dia_canvas_element_relative_angle_from(p_ang.x,p_ang.y);
}


/*Función que entrega el ángulo en el que se encuentra
 * real 
 * conectada una línea con respecto al ítem
 * Se le pasa el handle de la línea
 *
*/





/*
 * update_center_point:
 * @DiaCanvasItem: *item
 * centro del ítem cada vez que se actualiza éste.
 * Es de utilidad para objetos diferentes a líneas
 * Es el centro cartesiano
 * traslación del origen del plano de coordenadas
*/
static void update_center_point (DiaCanvasItem *item)
 {
  DiaCanvasElement* _item=(DiaCanvasElement*)item;
  item->origo.x=item->affine[4]+_item->width/2;
  item->origo.y=item->affine[5]+_item->height/2;
//  printf("update_center_point origen x, y : %f, %f \n",item->origo.x,item->origo.y);
 }



/*puts the new angle connex on list
 *
 */
static void save_dia_angleconex_point(DiaCanvasItem *item,DiaAngleConex* punto_conx)
{
  double x_max, y_max;
  /*
   *obtain the x_max & y_max relative from the item
  */
  x_max=item->origo.x-item->affine[4];
  y_max=item->origo.y-item->affine[5];

  /*obtain the original imaginary ro*/
  punto_conx->ro=dia_canvas_relative_hypotenusa_from(x_max,y_max); 
  punto_conx->ro_real=punto_conx->ro;
 /*obtain the y and x place from ro_real*/
//  printf("angulo real, %d \n",punto_conx->angulus_real);

   punto_conx->x=punto_conx->ro_real*cos(punto_conx->angulus_real*M_PI/180);
 /*obtain the y and y place from ro_real*/
   punto_conx->y=punto_conx->ro_real*sin(punto_conx->angulus_real*M_PI/180);

// printf("Punto x, punto y : %f, %f \n",punto_conx->x,punto_conx->y);

 /*original relative points are saved*/ 
   punto_conx->yo=punto_conx->y;
   punto_conx->xo=punto_conx->x;
 /*xmax & ymax orignal saved*/
    punto_conx->xmo=x_max;
    punto_conx->ymo=y_max;
 /*xmax var & ymasx var saved*/ 
    punto_conx->xm=x_max;
    punto_conx->ym=y_max;
 /*guarda el punto de conexión*/

 item->conxpoint=g_list_append(item->conxpoint,punto_conx);
}

/*
 * guarda la desviación o tolerancia de un punto de conex dado
 */
static void save_dia_angleconex_point_tol(DiaAngleConex* punto_conx, int toleran)
{
  punto_conx->torelantia=toleran;
}

/*
 *borra un DiaAngleConex* según ángulo de conexión
 * real
 **/
static void del_dia_angle_conex_point_r(DiaCanvasItem *item,int angle)
{
    DiaAngleConex* punto_conx=NULL;
    punto_conx=get_dia_angle_conex_r(item,angle);
    item->conxpoint=g_list_remove(item->conxpoint,punto_conx);
}

/*
 *borra un DiaAngleConex* según ángulo de conexión
 * usuario
 **/
static void del_dia_angle_conex_point_u(DiaCanvasItem *item,int angle)
{
    DiaAngleConex* punto_conx=NULL;
    punto_conx=get_diaangle_conex_u(item,angle);
    item->conxpoint=g_list_remove(item->conxpoint,punto_conx);
 }



/*
 * Actualiza los ángulos cuando al objeto se le 
 * cambia de tamaño
 * Sólo los ángulos de referencia
*/
static void update_dia_element_angle_conex(DiaCanvasItem* item)
{
 int cant, i,actua;
 double x_max, y_max,ro;
 double xvar,yvar;
 double tmp;
 GList* node;
 DiaAngleConex* punto_conx=NULL;
 node=NULL;
 actua=0;
 /*sets on 1 :)*/
 xvar=yvar=1.0;
 /*obtiene la cantidad de elementos*/
 cant=g_list_length(item->conxpoint); 
  /*
   *obtain the x_max & y_max relative from the item
  */
  x_max=item->origo.x-item->affine[4];
  y_max=item->origo.y-item->affine[5];

  /*ro es el mismo para cualquier lado del círculo*/
  ro=dia_canvas_relative_hypotenusa_from(x_max,y_max); 
  /*se verifica si es necesario actualizar
   * si lo es, calcula la varianza en x e y
  */
 if(cant>0)
  {
   node=g_list_nth(item->conxpoint,0);
   punto_conx=node->data;
     if((x_max!=punto_conx->xm)||(y_max!=punto_conx->ym ))  /*size change it?*/
        {
         actua=1;                 /*yes*/
         xvar=(double)x_max/punto_conx->xm;
         yvar=(double)y_max/punto_conx->ym;
	/*actualiza el máx x e y*/
	 punto_conx->xm=x_max;
	 punto_conx->ym=y_max;   
//         printf("actualizando angulos %f  %f \n",xvar,yvar);
	}
     }
  /*
   * como x e y es proporcional a r, 
   * entonces cualquier cambio en ro
   * también es proporcional a x e y
   * y como  se está normalizando el ítem
   * a una circurferencia de radio =ro
   * con ro dependiendo de x natural, y natural
   * qué es diferente al x má y el y max de la circunderencia 
  */
   
 if(actua==1)
  for(i=0;i<cant;i++)
     {
      
      node=g_list_nth(item->conxpoint,i);
      punto_conx=node->data;
      punto_conx->ro_real=ro;
    /* se calcula el nuevo x, & el nuevo y
     * de acuerdo a la varianza en x  & y
     */
      punto_conx->x=punto_conx->x*xvar;
      punto_conx->y=punto_conx->y*yvar;
     /*
     *se calcula el nuevo ángulo
    */
      punto_conx->angulus_real=dia_canvas_element_relative_angle_from(punto_conx->x,punto_conx->y);
      printf("agulus real %d \n",punto_conx->angulus_real);
    }

 }


/*
 *Entrega un 0 sí el ángulo no existe
 * en en la posición pedida ó en su tolerancia.
 *Entrega un 1, sí el ángulo existe en la posición pedida
 * o en la tolerancia
*/


static int find_angle_conex_tolerantia_u(DiaCanvasItem *item, int ang,int tol)
{
 int cant, i,exist;
 int ang_b,ang_a;
 int ang_b_p,ang_a_p;
 GList* node;
 DiaAngleConex* angulo_conex=NULL;
 exist=0;  /*por defecto entrega 0*/
/*obtiene la cantidad de elementos*/
 cant=g_list_length(item->conxpoint); 
 for(i=0;i<cant;i++)
   {
    node=g_list_nth(item->conxpoint,i);
    angulo_conex=(DiaAngleConex*)node->data;
    if(angulo_conex->angulus==ang)
      {
       exist=1;
       break;
      }
   /*Busca los ángulos de conexión de tolerancia oficial*/
    ang_b=angulo_conex->angulus-angulo_conex->torelantia;
    if(ang_b<0)
       ang_b+=360;
    ang_a=angulo_conex->angulus+angulo_conex->torelantia;
    if(ang_a>359)
       ang_a=ang_a-360;
   /*Busca los ángulos de conexión de tolerancia respecto al ángulo de pregunta */
    ang_b_p=ang-tol;
    if(ang_b_p<0)
       ang_b_p+=360;
    ang_a_p=ang+tol;
    if(ang_a_p>359)
       ang_a_p=ang_a-360;
   /*Sí el ańgulo de ref es mayor al ángulo de la pregunta*/
    if(angulo_conex->angulus > ang)
	{
          if(ang_b<=ang_a_p) /*es menor, la parte baja del ángulo a  la parte alta del otro*/ 
	    exist=2;
          else
            exist=0;
          break;
       }else
    if(ang > angulo_conex->angulus)
	{
   	  if(ang_a>=ang_b_p) /*es menor, la parte baja del ángulo a  la parte alta del otro*/ 
	    exist=3;
          else
            exist=0;
          break;
        }
     }
  return exist;
  }

/*
 *Entrega un 0 sí 
 * No hay opción de conexión 
 * en en la posición pedida ó en su tolerancia.
 *Entrega un 1, sí el ángulo existe en la posición pedida
 * o en la tolerancia
 * Posibilidad de conexión en la ubicación pedida
 * Sí no tiene selección de ángulos, entonces entrega 1
*/

static int find_angle_conex_tolerantia_r(DiaCanvasItem *item, int ang)
{
 int cant, i,exist;
 int ang_b,ang_a;
 GList* node;
 DiaAngleConex* angulo_conex=NULL;
 exist=0;  /*por defecto entrega 0*/
/*obtiene la cantidad de elementos*/
 cant=g_list_length(item->conxpoint); 
 if(cant==0)
    exist=1;
 for(i=0;i<cant;i++)
   {
    node=g_list_nth(item->conxpoint,i);
    angulo_conex=(DiaAngleConex*)node->data;
   /*Busca los ángulos de conexión de tolerancia oficial*/
    ang_b=angulo_conex->angulus_real-angulo_conex->torelantia;
    if(ang_b<0)
       ang_b+=360;
    ang_a=angulo_conex->angulus_real+angulo_conex->torelantia;
    if(ang_a>359)
       ang_a=ang_a-360;
 /*¿está dentro del rango?*/
    if((ang<=ang_a)&&(ang>=ang_b))/*¿está dentro del rango?*/
       {
        exist=1;
         break;  /*si*/
       }
     }
  return exist;
  }


/*
 * Entrega el ángulo
 * de refrencia de acuerdo a un ángulo
 * de tolerancia
 * -1 si no lo encuentra
*/

static int find_angle_conex_ref_from_tol(DiaCanvasItem *item, int ang)
{
 int cant, i,exist;
 int ang_b,ang_a;
 GList* node;
 int angle_ref=-1;
 DiaAngleConex* angulo_conex=NULL;
 exist=0;  /*por defecto entrega 0*/
/*obtiene la cantidad de elementos*/
 cant=g_list_length(item->conxpoint); 
 if(cant==0)
    return angle_ref;
 for(i=0;i<cant;i++)
   {
    node=g_list_nth(item->conxpoint,i);
    angulo_conex=(DiaAngleConex*)node->data;
   /*Busca los ángulos de conexión de tolerancia oficial*/
    ang_b=angulo_conex->angulus_real-angulo_conex->torelantia;
    if(ang_b<0)
       ang_b+=360;
    ang_a=angulo_conex->angulus_real+angulo_conex->torelantia;
    if(ang_a>359)
       ang_a=ang_a-360;
 /*¿está dentro del rango?*/
    if((ang<=ang_a)&&(ang>=ang_b))/*¿está dentro del rango?*/
       {
        exist=1;
         break;  /*si*/
       }
     }

   if(exist)
    angle_ref=angulo_conex->angulus_real;
  return angle_ref;
  }




/*
 * entrega el flujo de datos
 * según el ángulo dado
 * Busca en los ángulos absolutos y no en los
 * relativos.
 * La dirección en que se puede conectar la línea, sí es entrada o salida
*/
static int find_conex_fluxus(DiaCanvasItem *item, int ang)
{
 int cant, i,exist;
 int ang_b,ang_a;
 GList* node;
 DiaAngleConex* angulo_conex=NULL;
 exist=0;  /*por defecto entrega 0*/
/*obtiene la cantidad de elementos*/
 cant=g_list_length(item->conxpoint); 
 if(cant==0)
    return 3;
 for(i=0;i<cant;i++)
   {
    node=g_list_nth(item->conxpoint,i);
    angulo_conex=(DiaAngleConex*)node->data;
   /*Busca los ángulos de conexión de tolerancia oficial*/
    ang_b=angulo_conex->angulus_real-angulo_conex->torelantia;
    if(ang_b<0)
       ang_b+=360;
    ang_a=angulo_conex->angulus_real+angulo_conex->torelantia;
    if(ang_a>359)
       ang_a=ang_a-360;
 /*¿está dentro del rango?*/
    if((ang<=ang_a)&&(ang>=ang_b))/*¿está dentro del rango?*/
       {
        exist=1;
         break;  /*si*/
       }
     }

  return angulo_conex->fluxus;
  }


/*
 *Entrega el DiaAngleConex de acuerdo al ángulo
 * real que se pase como parámetro
 */
static DiaAngleConex* get_dia_angle_conex_r(DiaCanvasItem *item,int ang)
{
 int cant, i;
 GList* node;
 DiaAngleConex* angulo_conex=NULL;
 /*obtiene la cantidad de elementos*/
 cant=g_list_length(item->conxpoint); 
 for(i=0;i<cant;i++)
   {
    node=g_list_nth(item->conxpoint,i);
    angulo_conex=(DiaAngleConex*)node->data;
    if(angulo_conex->angulus_real==ang)
       break;  
    }
  return angulo_conex;
 }


/*
 *Entrega el DiaAngleConex de acuerdo al ángulo
 * de usuario que se pase como parámetro.
 * el ángulo de usuario es aquel que 
 * el usario puso en un principio y que no cambia aún cambie el tamaño
 */
static DiaAngleConex* get_diaangle_conex_u(DiaCanvasItem *item,int ang)
{
 int cant, i;
 GList* node;
 DiaAngleConex* angulo_conex=NULL;
 /*obtiene la cantidad de elementos*/
 cant=g_list_length(item->conxpoint); 
 for(i=0;i<cant;i++)
   {
    node=g_list_nth(item->conxpoint,i);
    angulo_conex=(DiaAngleConex*)node->data;
    if(angulo_conex->angulus==ang)
       break;  
    }
  return angulo_conex;
}

/*
 *Obtiene la lista de puntos de conexión to the item
 * Return the GList conexion_points
 */
static GList* get_dia_angle_conex_list(DiaCanvasItem *item)
{
 return item->conxpoint;
}


/*
 * relocalize_point_from:
 * @DiaCanvasItem: *item
 * @gdouble *x
 * @gdouble *y
 * Recibe  un par de puntos en coordenadas absolutas
 *   y los cambia de acuerdo a las coordenadas del ítem
 * traslación del origen del plano de coordenadas
*/
static void dia_canvas_item_relocalize_point_from(DiaCanvasItem *item,gdouble *x,gdouble *y)
 {
   *x=*x-item->origo.x;
  *y=(*y-item->origo.y)*(-1);
// printf("pos del cero x=%f ,y=%f \n",item->origo.x, item->origo.y);
 }



/*
 * dia_canvas_element_relative_angle_from:
 * @gdouble: x,
 * @gdouble: y
 * Entrega el ángulo relativo, de acuerdo 
 * a los parámetros entregados
 * Las coordenadas  recibidas tienen que ser relativas
 * Entrega un valor entero con el ángulo 0-360 grados
 *
*/
static int dia_canvas_element_relative_angle_from(double x, double y)
 {
  double ang_r;
  //entrega vallores entre -pi e pi
  ang_r=atan2(y,x);
  ang_r=ang_r*180/M_PI;
  if(ang_r<0)  /*rango de 0 a -pi*/
     ang_r=360+ang_r; /*le corrige*/
  return (int)ang_r;  /*hace un cambio de tipo .. :)*/
 }


