/***************************************************************************
 *            dia-canvas-punctum.c
 *        
 *
 *  Wed Sep 27 08:41:31 2006
 * PUNCTUM
 * Exell enrique Franklin Jiménez. 
 * Based upon dia-canvas-circle 
 * DiaCanvasPunctum
 * Copyright  2006  
 * 
 *****************************************************************************/
 
#include "dia-canvas-punctum.h"

#include "dia-shape.h"
#include <libart_lgpl/art_affine.h>
#include "dia-canvas-i18n.h"
#include "dia-canvas-groupable.h"
#include "dia-canvas-text.h"



enum {
	PROP_COLOR=1,		/*COLOR DEL PERÍMETRO*/
	PROP_PLENUS_COLOR,
        PROP_RADIUS,		/*El radio del puntoi*/	
	PROP_PERIMETROS_AMPLUS   /*ancho de la línea del perímetro*/


};

static void dia_canvas_punctum_class_init (DiaCanvasPunctumClass *klass);
static void dia_canvas_punctum_init (DiaCanvasPunctum *item);
static void dia_canvas_punctum_set_property (GObject *object,
					 guint property_id,
					 const GValue *value,
					 GParamSpec *pspec);
static void dia_canvas_punctum_get_property (GObject *object,
					 guint property_id,
					 GValue *value,
					 GParamSpec *pspec);
static void dia_canvas_punctum_dispose	(GObject *object);
static void dia_canvas_punctum_update	(DiaCanvasItem *item,
					 gdouble affine[6]);
static gboolean dia_canvas_punctum_get_shape_iter	(DiaCanvasItem *item,
						 DiaCanvasIter *iter);
static gboolean dia_canvas_punctum_shape_next	(DiaCanvasItem *item,
						 DiaCanvasIter *iter);
static DiaShape* dia_canvas_punctum_shape_value	(DiaCanvasItem *item,
						 DiaCanvasIter *iter);
static DiaCanvasElementClass *parent_class = NULL;

GType dia_canvas_punctum_get_type (void)
{
	static GType object_type = 0;

	if (!object_type) {
		static const GTypeInfo object_info = {
			sizeof (DiaCanvasPunctumClass),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			(GClassInitFunc) dia_canvas_punctum_class_init,
			(GClassFinalizeFunc) NULL,
			(gconstpointer) NULL, /* */
			sizeof (DiaCanvasPunctum),
			(guint16) 0, /*  */
			(GInstanceInitFunc) dia_canvas_punctum_init,
		};

		object_type = g_type_register_static (DIA_TYPE_CANVAS_ELEMENT,
						      "DiaCanvasPunctum",
						      &object_info, 0);
	}

	return object_type;
}


static void dia_canvas_punctum_class_init (DiaCanvasPunctumClass *klass)
{
	GObjectClass *object_class;
	DiaCanvasItemClass *item_class;
	object_class = (GObjectClass*) klass;
	item_class = DIA_CANVAS_ITEM_CLASS (klass);
	
	parent_class = g_type_class_peek_parent (klass);

	object_class->get_property = dia_canvas_punctum_get_property;
	object_class->set_property = dia_canvas_punctum_set_property;
	object_class->dispose = dia_canvas_punctum_dispose;

	item_class->update = dia_canvas_punctum_update;
	item_class->get_shape_iter = dia_canvas_punctum_get_shape_iter;
	item_class->shape_next = dia_canvas_punctum_shape_next;
	item_class->shape_value = dia_canvas_punctum_shape_value;
	
	g_object_class_install_property (object_class,
					 PROP_COLOR,
					 g_param_spec_ulong ("color",
						_("Color"),
						_("Color of the line in RGBA"),
						0, G_MAXULONG, 0,
						G_PARAM_READWRITE));

	g_object_class_install_property (object_class,
					 PROP_PLENUS_COLOR,
					 g_param_spec_ulong ("fill_color",
						_("Color de relleno"),
						_("Fill color"),
						0, G_MAXULONG, 0,
						G_PARAM_READWRITE));

	g_object_class_install_property (object_class,
					 PROP_PLENUS_COLOR,
					 g_param_spec_ulong ("plenus_color",
						_("Color de relleno"),
						_("Fill color"),
						0, G_MAXULONG, 0,
						G_PARAM_READWRITE));

	g_object_class_install_property (object_class,
					 PROP_PERIMETROS_AMPLUS,
					 g_param_spec_double ("border_width",
						"Ancho de la línea del perímetro",
						_("width of the border"),
						-G_MAXDOUBLE, G_MAXDOUBLE, 1.0,
						G_PARAM_READWRITE));
	g_object_class_install_property (object_class,
					 PROP_PERIMETROS_AMPLUS,
					 g_param_spec_double ("perimetros_amplus",
						"Ancho de la línea del perímetro",
						_("width of the border"),
						-G_MAXDOUBLE, G_MAXDOUBLE, 1.0,
						G_PARAM_READWRITE));


	g_object_class_install_property (object_class,
					 PROP_RADIUS,
					 g_param_spec_double ("radius",
						_("Radius punctum"),
						_("Radius' point"),
						0, G_MAXDOUBLE, 20,
						G_PARAM_READWRITE));

	}


static void dia_canvas_punctum_init (DiaCanvasPunctum *item)
{
       	item->color = DIA_COLOR(80, 80, 100);
	item->plenus_color = 0; /* fully transparent */
	item->perimetros_amplus =2.0;
	item->radius=10.0;
        g_object_set(item,"width",item->radius*2,NULL);
        g_object_set(item,"height",item->radius*2,NULL);  
        item->perimetros=dia_shape_new (DIA_SHAPE_ELLIPSE);
}



static void
dia_canvas_punctum_set_property (GObject *object, guint property_id,
			     const GValue *value, GParamSpec *pspec)
{
	DiaCanvasPunctum *punctum = (DiaCanvasPunctum*) object;
	switch (property_id) {
	case PROP_COLOR:
		dia_canvas_item_preserve_property (DIA_CANVAS_ITEM (punctum), "color");
		punctum->color = g_value_get_ulong (value);
		dia_canvas_item_request_update (DIA_CANVAS_ITEM (object));
		break;
	case PROP_PLENUS_COLOR:
		dia_canvas_item_preserve_property (DIA_CANVAS_ITEM(punctum), "plenus_color");
		punctum->plenus_color = g_value_get_ulong (value);
		dia_canvas_item_request_update (DIA_CANVAS_ITEM (object));
		break;
	case PROP_PERIMETROS_AMPLUS:
		dia_canvas_item_preserve_property (DIA_CANVAS_ITEM(punctum), "perimetros_amplus");
		punctum->perimetros_amplus = g_value_get_double (value);
		dia_canvas_item_request_update (DIA_CANVAS_ITEM (object));
		break;
       case PROP_RADIUS:
		dia_canvas_item_preserve_property (DIA_CANVAS_ITEM (punctum), "radius");
		punctum->radius = g_value_get_double (value);
		g_object_set(DIA_CANVAS_ITEM (punctum),"width",punctum->radius*2,NULL);
        	g_object_set(DIA_CANVAS_ITEM (punctum),"height",punctum->radius*2,NULL);  
		dia_canvas_item_request_update (DIA_CANVAS_ITEM (object));
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void dia_canvas_punctum_get_property (GObject *object, guint property_id,
			     GValue *value, GParamSpec *pspec)
{
	switch (property_id) {
	case PROP_COLOR:
		g_value_set_ulong (value, DIA_CANVAS_PUNCTUM (object)->color);
		break;
	case PROP_PLENUS_COLOR:
		g_value_set_ulong (value, DIA_CANVAS_PUNCTUM (object)->plenus_color);
		break;
	case PROP_PERIMETROS_AMPLUS:
		g_value_set_double (value, DIA_CANVAS_PUNCTUM (object)->perimetros_amplus);
		break;
       case PROP_RADIUS:
		g_value_set_double (value, DIA_CANVAS_PUNCTUM (object)->radius);
                break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void dia_canvas_punctum_dispose (GObject *object)
{
	DiaCanvasPunctum *punctum = (DiaCanvasPunctum*) object;

	if (punctum->perimetros) 
         {
		dia_shape_free (punctum->perimetros);
		punctum->perimetros = NULL;
	 }

	G_OBJECT_CLASS (parent_class)->dispose (object);
}


static void dia_canvas_punctum_update (DiaCanvasItem *item, gdouble affine[6])
{
	DiaCanvasPunctum* punctum;
	DiaShape *border;
        gdouble half_w;
        DiaPoint center,p;
	DiaPoint point_s[12]; 

       	//punctum->radius=20.0;  /*ancho*/
	punctum=(DiaCanvasPunctum*)item;

	punctum->area=M_PI*punctum->radius*punctum->radius;
        g_object_set(item,"width",punctum->radius*2,NULL);
        g_object_set(item,"height",punctum->radius*2,NULL);  

        if (DIA_CANVAS_ITEM_CLASS (parent_class)->update)
		DIA_CANVAS_ITEM_CLASS (parent_class)->update (item, affine);

	center.x = punctum->radius;
	center.y = punctum->radius;
	dia_shape_ellipse (punctum->perimetros,&center,punctum->radius*2,punctum->radius*2);
	dia_shape_ellipse_set_line_width (punctum->perimetros,
                                       DIA_CANVAS_PUNCTUM (item)->perimetros_amplus);
	dia_shape_set_color (punctum->perimetros, DIA_CANVAS_PUNCTUM (item)->color);
	dia_shape_ellipse_set_fill (punctum->perimetros, DIA_FILL_SOLID);
	dia_shape_ellipse_set_fill_color (punctum->perimetros,
                                          DIA_CANVAS_PUNCTUM (item)->plenus_color);
        dia_shape_ellipse_set_clipping  (punctum->perimetros,
                                             FALSE);

       

       dia_shape_request_update (punctum->perimetros);

	

}

static gboolean dia_canvas_punctum_get_shape_iter (DiaCanvasItem *item, DiaCanvasIter *iter)
{
       	iter->data[0] = DIA_CANVAS_PUNCTUM (item)->perimetros;
	return TRUE;
}

static gboolean
dia_canvas_punctum_shape_next (DiaCanvasItem *item, DiaCanvasIter *iter)
{
       	iter->data[0] = NULL;
	return FALSE;
}

static DiaShape*
dia_canvas_punctum_shape_value (DiaCanvasItem *item, DiaCanvasIter *iter)
{
	return iter->data[0];
}
